


type OnConnectionLostHandler = (error: MQTTError) => void;
export  type Ilang = 'zh-CN' | 'en-US';
export  type IOutput =  'publish_attribute' | 'report' | 'invoke_result' | 'unkonw';

interface MQTTError {
  /** A number indicating the nature of the error. */
  errorCode: number;

  /** Text describing the error */
  errorMessage: string;
}
export const enum gatewayOperation {
  search, // search
  subDevice = 1, // 返回子设备
  cancelSearch = 3 // cancelSearch
}

interface IProps {
  host: string;
  clientId: string;
  userName: string;
  password: string;
  lang?: Ilang;
  onConnectionLost: OnConnectionLostHandler;
  onMessageArrived: (message: IMessage) => void;
}

interface IMessage {
  type: string;
  data: string;
  deviceId: string;
  name?: string;
}

interface  IReport {
  deviceId: string ;
  event_name: string
}

interface Invoke {
  deviceId: string ;
  service_name: string
}

interface ISend {
  deviceId: string ;
  service_name: string;
  params: string
}
interface IGateway {
  deviceId: string;
  type: gatewayOperation
  params?: string
}
interface IUnlinkGateway {
  device: string
  params: {
    productId: string
    mac: string
  }
}

type connect = ErrorWithInvocationContext | void
type sub = OnSubscribeSuccessParams | ErrorWithInvocationContext

interface OnSubscribeSuccessParams extends WithInvocationContext {
  grantedQos: number;
}

interface WithInvocationContext {
  /**
   * <code>invocationContext</code> as passed in with the corresponding field in the connectOptions or
   * subscribeOptions.
   */
  invocationContext: any;
}
interface ErrorWithInvocationContext extends MQTTError, WithInvocationContext {
}

interface IAttr {
  deviceId: string
}

declare module 'xlink-jssdk' {
  export default class Xsdk {
    constructor(props: IProps)  //构造函数
    public connect(): Promise<connect> // 共有方法
    public disconnect(): void
    public subscribe(topic: string): Promise<sub >
    public unsubscribe(topic: string): Promise<sub>
    public publishAttr(params: IAttr): Promise<sub>
    public eventReport(params: IReport): Promise<sub>
    public serviceInvoke(params: Invoke): Promise<sub>
    public log(params: IAttr): Promise<sub>

    public unsubLog(params: IAttr): Promise<sub>
    public unsubPublishAttr(params: IAttr): Promise<sub>
    public unsubEventReport(params: IReport): Promise<sub>
    public unsubServiceInvoke(params: Invoke): Promise<sub>

    public invoke(params: ISend): void
    public invokeGateway(params: IGateway): void
    public unlinkGateway(params: IUnlinkGateway): void
  }
}

